/* This class will manage both the current image displayed on the screen as well as the thumbnail images. */

#ifndef __IMAGE_MANAGER_H
#define __IMAGE_MANAGER_H

#include "Image.h"
#include "ImageFileFormat.h"
#include <map>

#define CACHE_SIZE  27

typedef std::map<std::string, blepo::ImgBgr *> ImageMap;
typedef pair <std::string, blepo::ImgBgr*> key_pair;

class ImageManager
{
public:
  ImageManager();
  ~ImageManager();
        
  // must be called before anything else, so that it knows how to load
  // images from disk.  Uses CFileFind to scan the directory to build an
  // internal std::map from t/w/s to image filename.

  // Run at the startup of the program (Probably during OnFileNew in RootflyDlg.cpp)
  void SetDirectoryAndImageFileFormat(const CString& dir, const ImageFileFormat& iff);
  
  // Tells the class the current t/w/s.  This helps the class to know
  // which images / thumbnails to keep around in its cache.
  void SetCurrentTws(const CString& tube, int window, int session);

  //For this function we will use a hash table (std::map), and retrive the main image from the map using tws concatenated as the key
  //Also sets the main image in the display
  void GetImage(const CString& tube, int window, int session, blepo::ImgBgr *out) const;

  //This function will do much the same as GetImage, except for it will handle the thumbnails
  void GetThumbnail(const CString& tube, int window, int session, blepo::ImgBgr *out) const;

  // Allocate memory for image
  void AllocateMemory();

private:
  ImageMap m_image_map;                           // Hash table that holds all images found in folder
  ImageMap m_image_cache;                         // Hash table that holds common images (Recently used images)
  std::vector<blepo::ImgBgr *> m_image_vector;    // Holds pointers to the pictures
  blepo::ImgBgr *m_blank_image;                   // Used to allocate memory for pictures

  // Private method to store images in maps
  void StoreImage(const CString& tube, int window, int session, blepo::ImgBgr *image_ptr, ImageMap &image_map);
};

#endif