// ManageConditionCodesDlg.cpp : implementation file
//

#include "stdafx.h"
#include "rootfly.h"
#include "ManageConditionCodesDlg.h"
#include "AddConditionCodeDlg.h"
#include "MMConversionDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define INITIAL_DEFAULT "NO_CODE"

/////////////////////////////////////////////////////////////////////////////
// ManageConditionCodesDlg dialog


ManageConditionCodesDlg::ManageConditionCodesDlg(CWnd* pParent /*=NULL*/)
	: CDialog(ManageConditionCodesDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(ManageConditionCodesDlg)
	//}}AFX_DATA_INIT
  m_default_index = 0;
}


void ManageConditionCodesDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(ManageConditionCodesDlg)
	DDX_Control(pDX, IDC_BUTTON_UP, m_button_up);
	DDX_Control(pDX, IDC_BUTTON_DOWN, m_button_down);
	DDX_Control(pDX, IDC_TEXT_DEFAULT, m_text_default);
	DDX_Control(pDX, IDC_BUTTON_DELETE, m_button_delete);
	DDX_Control(pDX, IDC_BUTTON_ADD, m_button_add);
	DDX_Control(pDX, IDC_BUTTON_DEFAULT, m_button_default);
	DDX_Control(pDX, IDC_LIST, m_list);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(ManageConditionCodesDlg, CDialog)
	//{{AFX_MSG_MAP(ManageConditionCodesDlg)
	ON_LBN_SELCHANGE(IDC_LIST, OnSelchangeList)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnButtonDelete)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_DEFAULT, OnButtonDefault)
	ON_BN_CLICKED(IDC_CANCEL, OnCancel)
	ON_BN_CLICKED(IDC_BUTTON_DOWN, OnButtonDown)
	ON_BN_CLICKED(IDC_BUTTON_UP, OnButtonUp)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// ManageConditionCodesDlg message handlers
void ManageConditionCodesDlg::SetConditionCodes(std::vector<CString> user_codes)
{
  m_codes = user_codes;
}


void ManageConditionCodesDlg::OnOK() 
{
  CString temp;
  m_codes.clear();
  for(int i = 0; i < m_list.GetCount();i++)
  {
    m_list.GetText(i,temp);
    m_codes.push_back(temp);
  }

	CDialog::OnOK();
}

void ManageConditionCodesDlg::OnSelchangeList() 
{
  EnableDisable();
}

void ManageConditionCodesDlg::OnButtonDelete() 
{
	CString temp = "";
  CString default_str = INITIAL_DEFAULT;
  int index = m_list.GetCurSel();
  if(index >= 0)
  {
    if(index <= m_default_index)
    {
      if(m_list.GetCount() == 1)
      {
        m_default_index = 0;
        m_list.DeleteString(0);
        m_list.AddString(default_str);
        UpdateDefault();
      } 
      else
      {
        if (index == m_default_index && m_default_index == 0) 
        {
          m_list.DeleteString(index);
          UpdateDefault();
          m_list.GetText(m_default_index,temp);
        } 
        else
        {
          // default deleted, notify user
          if(index == m_default_index)
          {
            assert( m_default_index > 0 );
            m_default_index--;
            m_list.DeleteString(index);
            UpdateDefault();
            m_list.GetText(m_default_index,temp);
          } 
          else
          {
            assert( m_default_index > 0 );
            m_default_index--;
            m_list.DeleteString(index);
            UpdateDefault();
          }
        }
      }
    } 
    else 
    {
      m_list.DeleteString(index);
    }
    m_list.SetCurSel( index < m_list.GetCount() ? index : m_list.GetCount()-1 );
  }	
  EnableDisable();
}

// Returns true if the new code is added, false otherwise
bool ManageConditionCodesDlg::AddNewCode(const CString& new_code)
{
  bool added;

  int index = m_list.FindStringExact(-1, new_code);
  if (index >= 0)
  {
    // if the code already exists, then do not add it
    CString str;
    str.Format("Code '%s' already exists.", new_code);
    AfxMessageBox(str, MB_ICONWARNING);
    m_list.SetCurSel(index);
    added = false;
  } 
  else 
  {
    // Check to make sure that the code is legal.
    // (Empty strings and leading whitespace are not allowed, because this would
    //  prevent the .rfy file from being loaded properly (since we use fgets, 
    //  which eats leading whitespace).)  
    if (new_code == "")
    {
      AfxMessageBox("The empty string is not a valid code.");
      added = false;
    }
    else if (new_code.GetLength() > 0 && new_code[0] == ' ')
    {
      AfxMessageBox("Codes are not allowed to begin with a space.", MB_ICONWARNING);
      added = false;
    }
    else
    {
      m_list.AddString(new_code);
      index = m_list.GetCount() - 1;
      m_list.SetCurSel(index);
      added = true;
    }
  }
  return added;
}

void ManageConditionCodesDlg::OnButtonAdd() 
{
  CString new_code;
  bool done;

  do
  {
    AddConditionCodeDlg dlg;
    if (dlg.DoModal() == IDOK)
    {
      new_code = dlg.GetAddedCode();
      done = AddNewCode(new_code);
    }
    else
    {
      done = true;
    }
  }
  while (!done);
  EnableDisable();
}

BOOL ManageConditionCodesDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  m_button_down.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_DOWN));
  m_button_up.SetIcon(AfxGetApp()->LoadIcon(IDI_ICON_UP));

  CString temp = " ";
  for (int i = 0; i < m_codes.size();i++)
  {
    m_list.AddString(m_codes.at(i));
  }	

  // set currect selection 
  // (Really should set it to default)
  if (m_codes.size() > 0)  m_list.SetCurSel(0);

  EnableDisable();
  UpdateDefault();  
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void ManageConditionCodesDlg::OnCancel() 
{
  CDialog::OnCancel();	
}


void ManageConditionCodesDlg::UpdateDefault() 
{
  CString temp;
  assert( m_default_index >= 0 && m_default_index  < m_list.GetCount());
  if ( m_default_index < 0 || m_default_index  >= m_list.GetCount())  return;
  m_list.GetText(m_default_index, temp);
  m_text_default.SetWindowText(CString("Default: ") + temp);
}

void ManageConditionCodesDlg::OnButtonDefault() 
{
	CString temp;
  int index = m_list.GetCurSel();
  if(index >= 0)
  {
    m_default_index = index;
    UpdateDefault();
  }	
}

void ManageConditionCodesDlg::EnableDisable()
{
  if(m_list.GetCurSel() < 0)
  {
    m_button_default.EnableWindow(false);
    m_button_delete.EnableWindow(false);
  } else
  {
    m_button_default.EnableWindow(true);
    m_button_delete.EnableWindow(true);
  }
}

void ManageConditionCodesDlg::OnButtonUp() 
{
  CString temp_str;
	int cur = m_list.GetCurSel();
  if(m_list.GetCount() > 1 && cur > 0)
  {
    if(cur-1 == m_default_index) m_default_index = cur;
    else if(cur == m_default_index) m_default_index--;
    m_list.GetText(cur,temp_str);
    m_list.DeleteString(cur);
    m_list.InsertString(cur-1,temp_str);
    m_list.SetCurSel(cur-1);
  }	
}

void ManageConditionCodesDlg::OnButtonDown() 
{
  CString temp_str;
	int cur = m_list.GetCurSel();
  if(m_list.GetCount() > 1 && cur >= 0 && cur <= m_list.GetCount()-2)
  {
    if(cur+1 == m_default_index) m_default_index = cur;
    else if(cur == m_default_index) m_default_index++;
    m_list.GetText(cur,temp_str);
    m_list.DeleteString(cur);
    m_list.InsertString(cur+1,temp_str);
    m_list.SetCurSel(cur+1);
  }	
}
